# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url
from django.contrib import admin
from django.db.models import Q
from django.http import JsonResponse, HttpResponseForbidden
from django.utils.translation import get_language_from_request
from cms.models.pagemodel import Page
from cms.extensions import PageExtensionAdmin
from cmsplugin_cascade.models import CascadePage
from cmsplugin_cascade.link.forms import format_page_link


@admin.register(CascadePage)
class CascadePageAdmin(PageExtensionAdmin):
    change_form_template = 'cascade/admin/cascadepage_change_form.html'

    def get_fields(self, request, obj=None):
        return ['icon_font']

    def get_urls(self):
        urls = [
            url(r'^get_page_sections/$', lambda: None, name='get_page_sections'),  # just to reverse
            url(r'^get_page_sections/(?P<page_pk>\d+)$',
                self.admin_site.admin_view(self.get_page_sections)),
            url(r'^published_pages/$', self.get_published_pagelist, name='get_published_pagelist'),
        ]
        urls.extend(super(CascadePageAdmin, self).get_urls())
        return urls

    def get_page_sections(self, request, page_pk=None):
        choices = []
        try:
            for key, val in self.model.objects.get(extended_object_id=page_pk).glossary['element_ids'].items():
                choices.append((key, val))
        except (self.model.DoesNotExist, KeyError):
            pass
        return JsonResponse({'element_ids': choices})

    def get_published_pagelist(self, request, *args, **kwargs):
        """
        This view is used by the SearchLinkField as the user type to feed the autocomplete drop-down.
        """
        if not request.is_ajax():
            return HttpResponseForbidden()

        query_term = request.GET.get('term','').strip('/')
        language = get_language_from_request(request)
        matching_published_pages = Page.objects.published().public().filter(
            Q(title_set__title__icontains=query_term, title_set__language=language)
            | Q(title_set__path__icontains=query_term, title_set__language=language)
            | Q(title_set__menu_title__icontains=query_term, title_set__language=language)
            | Q(title_set__page_title__icontains=query_term, title_set__language=language)
        ).distinct().order_by('title_set__title').iterator()

        data = {'results': []}
        for page in matching_published_pages:
            title = page.get_title(language=language)
            path = page.get_absolute_url(language=language)
            data['results'].append({
                'id': page.pk,
                'text': format_page_link(title, path),
            })
            if len(data['results']) > 15:
                break
        return JsonResponse(data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    from html.parser import HTMLParser  # py3
except ImportError:
    from HTMLParser import HTMLParser  # py2

from django.forms import widgets
from django.utils.functional import cached_property
from django.utils.html import format_html
from django.utils.text import mark_safe
from django.utils.translation import ugettext_lazy as _
from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.fields import GlossaryField
from cmsplugin_cascade.plugin_base import TransparentContainer
from cmsplugin_cascade.bootstrap4.plugin_base import BootstrapPluginBase

html_parser = HTMLParser()


class BootstrapCardMixin(object):
    @cached_property
    def card_header(self):
        return mark_safe(html_parser.unescape(self.glossary.get('header', '')))

    @cached_property
    def no_body_padding(self):
        return not self.glossary.get('body_padding', True)

    @cached_property
    def card_footer(self):
        return mark_safe(html_parser.unescape(self.glossary.get('footer', '')))


class BootstrapCardPlugin(TransparentContainer, BootstrapPluginBase):
    """
    Use this plugin to display a card with optional card-header and card-footer.
    """
    name = _("Card")
    default_css_class = 'card'
    require_parent = False
    parent_classes = ['BootstrapColumnPlugin']
    model_mixins = (BootstrapCardMixin,)
    allow_children = True
    child_classes = None
    render_template = 'cascade/bootstrap4/card.html'
    glossary_field_order = ['header', 'body_padding', 'footer']

    header = GlossaryField(
        widgets.TextInput(attrs={'size': 80}),
        label=_("Card Header")
    )

    body_padding = GlossaryField(
         widgets.CheckboxInput(),
         label=_("Body with padding"),
         initial=True,
         help_text=_("Add standard padding to card body.")
    )

    footer = GlossaryField(
        widgets.TextInput(attrs={'size': 80}),
        label=_("Card Footer")
    )

    @classmethod
    def get_identifier(cls, obj):
        identifier = super(BootstrapCardPlugin, cls).get_identifier(obj)
        try:
            return format_html('{0}{1}', identifier, obj.card_header or obj.card_footer)
        except AttributeError:
            pass
        return identifier

plugin_pool.register_plugin(BootstrapCardPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.apps import apps
from django.forms import fields
from django.forms.models import ModelForm
from django.forms.widgets import Select as SelectWidget
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _, get_language
from cms.models import Page
from cmsplugin_cascade.utils import validate_link

try:
    from cms.utils import get_current_site
except ImportError:
    def get_current_site():
        from django.contrib.sites.models import Site
        return Site.objects.get_current()


def format_page_link(*args, **kwargs):
    return format_html("{} ({})", *args, **kwargs)


if 'django_select2' in settings.INSTALLED_APPS:
    from django_select2.forms import HeavySelect2Widget

    class HeavySelectWidget(HeavySelect2Widget):
        @property
        def media(self):
            parent_media = super(HeavySelectWidget, self).media
            # prepend JS snippet to re-add 'jQuery' to the global namespace
            parent_media._js.insert(0, 'cascade/js/admin/jquery.restore.js')
            return parent_media

        def render(self, name, value, attrs=None):
            try:
                page = Page.objects.get(pk=value)
            except (Page.DoesNotExist, ValueError):
                pass
            else:
                language = get_language()
                text = format_page_link(page.get_title(language), page.get_absolute_url(language))
                self.choices.append((value, text))
            html = super(HeavySelectWidget, self).render(name, value, attrs=attrs)
            return html


class LinkSearchField(fields.ChoiceField):
    def __init__(self, *args, **kwargs):
        if 'django_select2' in settings.INSTALLED_APPS:
            widget = HeavySelectWidget(data_view='admin:get_published_pagelist')
        else:
            widget = SelectWidget
        kwargs.setdefault('widget', widget)
        super(LinkSearchField, self).__init__(*args, **kwargs)

    def clean(self, value):
        try:
            return int(value)
        except (TypeError, ValueError):
            pass


class LinkForm(ModelForm):
    """
    Form class to add fake fields for rendering the ModelAdmin's form, which later are used to
    populate the glossary of the model.
    """
    LINK_TYPE_CHOICES = [('cmspage', _("CMS Page")), ('exturl', _("External URL")), ('email', _("Mail To"))]
    link_type = fields.ChoiceField(
        label=_("Link"),
        help_text=_("Type of link"),
    )

    cms_page = LinkSearchField(
        required=False,
        label='',
        help_text=_("An internal link any CMS page of this site"),
    )

    section = fields.ChoiceField(
        required=False,
        label='',
        help_text=_("Page bookmark"),
    )

    ext_url = fields.URLField(
        required=False,
        label='',
        help_text=_("Link onto external page"),
    )

    mail_to = fields.EmailField(
        required=False,
        label='',
        help_text=_("Open Email program with this address"),
    )

    class Meta:
        fields = ('glossary',)

    def __init__(self, data=None, *args, **kwargs):
        instance = kwargs.get('instance')
        default_link_type = {'type': self.LINK_TYPE_CHOICES[0][0]}
        initial = dict(instance.glossary) if instance else {'link': default_link_type}
        initial.update(kwargs.pop('initial', {}))
        initial.setdefault('link', {'type': default_link_type})
        link_type = initial['link']['type']
        self.base_fields['link_type'].choices = self.LINK_TYPE_CHOICES
        self.base_fields['link_type'].initial = link_type
        if data and data.get('shared_glossary'):
            # convert this into an optional field since it is disabled with ``shared_glossary`` set
            self.base_fields['link_type'].required = False
        set_initial_linktype = getattr(self, 'set_initial_{}'.format(link_type), None)
        if 'django_select2' not in settings.INSTALLED_APPS:
            # populate classic Select field for choosing a CMS page
            site = get_current_site()
            choices = [(p.pk, format_page_link(p.get_page_title(), p.get_absolute_url()))
                       for p in Page.objects.drafts().on_site(site)]
            self.base_fields['cms_page'].choices = choices

        if callable(set_initial_linktype):
            set_initial_linktype(initial)
        self._preset_section(data, initial)
        super(LinkForm, self).__init__(data, initial=initial, *args, **kwargs)

    def _preset_section(self, data, initial):
        choices = [(None, _("Page root"))]
        try:
            if data:
                cascade_page = Page.objects.get(pk=data['cms_page']).cascadepage
            else:
                cascade_page = Page.objects.get(pk=initial['link']['pk']).cascadepage
        except (KeyError, ValueError, ObjectDoesNotExist):
            pass
        else:
            for key, val in cascade_page.glossary.get('element_ids', {}).items():
                choices.append((key, val))

        self.base_fields['section'].initial = initial['link'].get('section')
        self.base_fields['section'].choices = choices

    def clean_glossary(self):
        """
        This method rectifies the behavior of JSONFormFieldBase.clean which converts
        the value of empty fields to None, although it shall be an empty dict.
        """
        glossary = self.cleaned_data['glossary']
        if glossary is None:
            glossary = {}
        return glossary

    def clean(self):
        cleaned_data = super(LinkForm, self).clean()
        if self.is_valid():
            if 'link_data' in cleaned_data:
                cleaned_data['glossary'].update(link=cleaned_data['link_data'])
                del self.cleaned_data['link_data']
            elif 'link_type' in cleaned_data:
                cleaned_data['glossary'].update(link={'type': cleaned_data['link_type']})
            else:
                cleaned_data['glossary'].update(link={'type': 'none'})
        return cleaned_data

    def clean_cms_page(self):
        if self.cleaned_data.get('link_type') == 'cmspage':
            self.cleaned_data['link_data'] = {
                'type': 'cmspage',
                'model': 'cms.Page',
                'pk': self.cleaned_data['cms_page'],
            }
            validate_link(self.cleaned_data['link_data'])
        return self.cleaned_data['cms_page']

    def clean_section(self):
        if self.cleaned_data.get('link_type') == 'cmspage':
            self.cleaned_data['link_data']['section'] = self.cleaned_data['section']
        return self.cleaned_data['section']

    def clean_ext_url(self):
        if self.cleaned_data.get('link_type') == 'exturl':
            self.cleaned_data['link_data'] = {'type': 'exturl', 'url': self.cleaned_data['ext_url']}
        return self.cleaned_data['ext_url']

    def clean_mail_to(self):
        if self.cleaned_data.get('link_type') == 'email':
            self.cleaned_data['link_data'] = {'type': 'email', 'email': self.cleaned_data['mail_to']}
        return self.cleaned_data['mail_to']

    def set_initial_none(self, initial):
        pass

    def set_initial_cmspage(self, initial):
        try:
            # check if that page still exists, otherwise return nothing
            Model = apps.get_model(*initial['link']['model'].split('.'))
            initial['cms_page'] = Model.objects.get(pk=initial['link']['pk']).pk
        except (KeyError, ObjectDoesNotExist):
            pass

    def set_initial_exturl(self, initial):
        try:
            initial['ext_url'] = initial['link']['url']
        except KeyError:
            pass

    def set_initial_email(self, initial):
        try:
            initial['mail_to'] = initial['link']['email']
        except KeyError:
            pass

    @classmethod
    def get_form_class(cls):
        """
        Hook to return a form class for editing a CMSPlugin inheriting from ``LinkPluginBase``.
        """
        return cls

    @classmethod
    def unset_required_for(cls, sharable_fields):
        """
        Fields borrowed by `SharedGlossaryAdmin` to build its temporary change form, only are
        required if they are declared in `sharable_fields`. Otherwise just deactivate them.
        """
        if 'link_content' in cls.base_fields and 'link_content' not in sharable_fields:
            cls.base_fields['link_content'].required = False
        if 'link_type' in cls.base_fields and 'link' not in sharable_fields:
            cls.base_fields['link_type'].required = False


class TextLinkFormMixin(object):
    """
    To be used in combination with `LinkForm` for easily accessing the field `link_content`.
    """
    def clean(self):
        cleaned_data = super(TextLinkFormMixin, self).clean()
        if self.is_valid():
            cleaned_data['glossary'].update(link_content=cleaned_data['link_content'])
        return cleaned_data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import warnings
from django.contrib.admin.utils import unquote
from django.utils.html import format_html, format_html_join
from django.utils.safestring import mark_safe
from cmsplugin_cascade.models import CascadePage
from cmsplugin_cascade.plugin_base import CascadePluginMixinBase


class IconModelMixin(object):
    @property
    def icon_font_attrs(self):
        icon_font = self.plugin_class.get_icon_font(self)
        symbol = self.glossary.get('symbol')
        attrs = []
        if icon_font and symbol:
            attrs.append(mark_safe('class="{}{}"'.format(icon_font.config_data.get('css_prefix_text', 'icon-'), symbol)))
        styles = {'display': 'inline-block'}
        disabled, color = self.glossary.get('color', (True, '#000000'))
        if not disabled:
            styles['color'] = color
        disabled, background_color = self.glossary.get('background_color', (True, '#000000'))
        if not disabled:
            styles['background-color'] = background_color
        border = self.glossary.get('border')
        if isinstance(border, list) and border[0] and border[1] != 'none':
            styles.update(border='{0} {1} {2}'.format(*border))
            radius = self.glossary.get('border_radius')
            if radius:
                styles['border-radius'] = radius
        attrs.append(format_html('style="{}"',
                                 format_html_join('', '{0}:{1};',
                                                  [(k, v) for k, v in styles.items()])))
        return mark_safe(' '.join(attrs))


class IconPluginMixin(CascadePluginMixinBase):
    change_form_template = 'cascade/admin/fonticon_plugin_change_form.html'
    ring_plugin = 'IconPluginMixin'
    require_icon_font = True  # if False, the icon_font is optional

    class Media:
        css = {'all': ['cascade/css/admin/iconplugin.css']}
        js = ['cascade/js/admin/iconpluginmixin.js']

    @classmethod
    def get_identifier(cls, instance):
        identifier = super(IconPluginMixin, cls).get_identifier(instance)
        icon_font = cls.get_icon_font(instance)
        if icon_font:
            symbol = mark_safe('<i class="{}{}"></i>'.format(
                icon_font.config_data.get('css_prefix_text', 'icon-'),
                instance.glossary.get('symbol')))
            return format_html('{0}{1}', identifier, symbol)
        return identifier

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        try:
            if object_id:
                icon_font = self.get_object(request, unquote(object_id)).cmsplugin_ptr.page.cascadepage.icon_font
            else:
                icon_font = self._cms_initial_attributes['placeholder'].page.cascadepage.icon_font
        except (AttributeError, CascadePage.DoesNotExist):
            icon_font = None
        extra_context = dict(extra_context or {}, icon_font=icon_font, require_icon_font=self.require_icon_font)
        return super(IconPluginMixin, self).changeform_view(
             request, object_id=object_id, form_url=form_url, extra_context=extra_context)

    @classmethod
    def get_icon_font(self, instance):
        try:
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                return instance.cmsplugin_ptr.page.cascadepage.icon_font
        except (CascadePage.DoesNotExist, AttributeError):
            return

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        icon_font = self.get_icon_font(instance)
        if icon_font:
            context['stylesheet_url'] = icon_font.get_stylesheet_url()
        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.forms import widgets, ModelChoiceField
from django.utils.html import format_html, format_html_join
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from cms.plugin_pool import plugin_pool
from filer.models.imagemodels import Image

from cmsplugin_cascade import app_settings
from cmsplugin_cascade.fields import GlossaryField
from cmsplugin_cascade.image import ImageAnnotationMixin, ImageFormMixin, ImagePropertyMixin
from cmsplugin_cascade.link.config import LinkPluginBase, LinkElementMixin, LinkForm
from cmsplugin_cascade.plugin_base import CascadePluginBase, TransparentContainer
from cmsplugin_cascade.utils import compute_aspect_ratio
from cmsplugin_cascade.widgets import CascadingSizeWidget


class SimpleWrapperPlugin(TransparentContainer, CascadePluginBase):
    name = _("Simple Wrapper")
    parent_classes = None
    require_parent = False
    allow_children = True
    alien_child_classes = True
    TAG_CHOICES = tuple((cls, _("<{}> – Element").format(cls))
        for cls in ('div', 'span', 'section', 'article',)) + (('naked', _("Naked Wrapper")),)

    tag_type = GlossaryField(
        widgets.Select(choices=TAG_CHOICES),
        label=_("HTML element tag"),
        help_text=_('Choose a tag type for this HTML element.')
    )

    @classmethod
    def get_identifier(cls, instance):
        identifier = super(SimpleWrapperPlugin, cls).get_identifier(instance)
        tag_name = dict(cls.TAG_CHOICES).get(instance.glossary.get('tag_type'))
        if tag_name:
            return format_html('{0} {1}', tag_name, identifier)
        return identifier

    def get_render_template(self, context, instance, placeholder):
        if instance.glossary.get('tag_type') == 'naked':
            return 'cascade/generic/naked.html'
        return 'cascade/generic/wrapper.html'

plugin_pool.register_plugin(SimpleWrapperPlugin)


class HorizontalRulePlugin(CascadePluginBase):
    name = _("Horizontal Rule")
    parent_classes = None
    allow_children = False
    tag_type = 'hr'
    render_template = 'cascade/generic/single.html'
    glossary_fields = ()

plugin_pool.register_plugin(HorizontalRulePlugin)


class HeadingPlugin(CascadePluginBase):
    name = _("Heading")
    parent_classes = None
    allow_children = False
    TAG_TYPES = tuple(('h{}'.format(k), _("Heading {}").format(k)) for k in range(1, 7))
    glossary_field_order = ['tag_type', 'content', 'element_id']

    tag_type = GlossaryField(widgets.Select(choices=TAG_TYPES))

    content = GlossaryField(
        widgets.TextInput(attrs={'style': 'width: 100%; padding-right: 0; font-weight: bold; font-size: 125%;'}),
        label=_("Heading content"))

    render_template = 'cascade/generic/heading.html'

    class Media:
        css = {'all': ('cascade/css/admin/partialfields.css',)}

    @classmethod
    def get_identifier(cls, instance):
        identifier = super(HeadingPlugin, cls).get_identifier(instance)
        tag_type = instance.glossary.get('tag_type')
        content = mark_safe(instance.glossary.get('content', ''))
        if tag_type:
            return format_html('<code>{0}</code>: {1} {2}', tag_type, content, identifier)
        return content

    def render(self, context, instance, placeholder):
        context = self.super(HeadingPlugin, self).render(context, instance, placeholder)
        context.update({'content': mark_safe(instance.glossary.get('content', ''))})
        return context

plugin_pool.register_plugin(HeadingPlugin)


class CustomSnippetPlugin(TransparentContainer, CascadePluginBase):
    """
    Allows to add a customized template anywhere. This plugins will be registered only if the
    project added a template using the configuration setting 'plugins_with_extra_render_templates'.
    """
    name = _("Custom Snippet")
    require_parent = False
    allow_children = True
    alien_child_classes = True
    render_template_choices = dict(app_settings.CMSPLUGIN_CASCADE['plugins_with_extra_render_templates'].get('CustomSnippetPlugin', ()))
    render_template = 'cascade/generic/does_not_exist.html'  # default in case the template could not be found

    @classmethod
    def get_identifier(cls, instance):
        render_template = instance.glossary.get('render_template')
        if render_template:
            return format_html('{}', cls.render_template_choices.get(render_template))


if CustomSnippetPlugin.render_template_choices:
    # register only, if at least one template has been defined
    plugin_pool.register_plugin(CustomSnippetPlugin)


class TextImagePlugin(ImageAnnotationMixin, LinkPluginBase):
    name = _("Image in text")
    text_enabled = True
    ring_plugin = 'TextImagePlugin'
    render_template = 'cascade/plugins/textimage.html'
    parent_classes = ('TextPlugin',)
    model_mixins = (ImagePropertyMixin, LinkElementMixin)
    allow_children = False
    require_parent = False
    html_tag_attributes = {'image_title': 'title', 'alt_tag': 'tag'}
    html_tag_attributes.update(LinkPluginBase.html_tag_attributes)
    fields = ['image_file'] + list(LinkPluginBase.fields)
    RESIZE_OPTIONS = [('upscale', _("Upscale image")), ('crop', _("Crop image")),
                      ('subject_location', _("With subject location")),
                      ('high_resolution', _("Optimized for Retina"))]

    image_width = GlossaryField(
        CascadingSizeWidget(allowed_units=['px'], required=True),
        label=_("Image Width"),
        help_text=_("Set the image width in pixels."),
    )

    image_height = GlossaryField(
        CascadingSizeWidget(allowed_units=['px'], required=False),
        label=_("Image Height"),
        help_text=_("Set the image height in pixels."),
    )

    resize_options = GlossaryField(
        widgets.CheckboxSelectMultiple(choices=RESIZE_OPTIONS),
        label=_("Resize Options"),
        help_text=_("Options to use when resizing the image."),
        initial=['subject_location', 'high_resolution']
    )

    alignement = GlossaryField(
        widgets.RadioSelect(choices=[('', _("Not aligned")), ('left', _("Left")), ('right', _("Right"))]),
        initial='',
        label=_("Alignement"),
    )

    class Media:
        js = ['cascade/js/admin/textimageplugin.js']

    def get_form(self, request, obj=None, **kwargs):
        LINK_TYPE_CHOICES = (('none', _("No Link")),) + \
            tuple(t for t in getattr(LinkForm, 'LINK_TYPE_CHOICES') if t[0] != 'email')
        image_file = ModelChoiceField(queryset=Image.objects.all(), required=False, label=_("Image"))
        Form = type(str('ImageForm'), (ImageFormMixin, getattr(LinkForm, 'get_form_class')(),),
                    {'LINK_TYPE_CHOICES': LINK_TYPE_CHOICES, 'image_file': image_file})
        kwargs.update(form=Form)
        return super(TextImagePlugin, self).get_form(request, obj, **kwargs)

    @classmethod
    def requires_parent_plugin(cls, slot, page):
        """
        Workaround for `PluginPool.get_all_plugins()`, otherwise TextImagePlugin is not allowed
        as a child of a `TextPlugin`.
        """
        return False

    @classmethod
    def get_inline_styles(cls, instance):
        inline_styles = cls.super(TextImagePlugin, cls).get_inline_styles(instance)
        alignement = instance.glossary.get('alignement')
        if alignement:
            inline_styles['float'] = alignement
        return inline_styles

    def render(self, context, instance, placeholder):
        try:
            aspect_ratio = compute_aspect_ratio(instance.image)
        except Exception:
            # if accessing the image file fails, abort here
            return context
        resize_options = instance.glossary.get('resize_options', {})
        crop = 'crop' in resize_options
        upscale = 'upscale' in resize_options
        subject_location = instance.image.subject_location if 'subject_location' in resize_options else False
        high_resolution = 'high_resolution' in resize_options
        image_width = instance.glossary.get('image_width', '')
        if not image_width.endswith('px'):
            return context
        image_width = int(image_width.rstrip('px'))
        image_height = instance.glossary.get('image_height', '')
        if image_height.endswith('px'):
            image_height = int(image_height.rstrip('px'))
        else:
            image_height = int(round(image_width * aspect_ratio))
        src = {
            'size': (image_width, image_height),
            'size2x': (image_width * 2, image_height * 2),
            'crop': crop,
            'upscale': upscale,
            'subject_location': subject_location,
            'high_resolution': high_resolution,
        }
        link_attributes = LinkPluginBase.get_html_tag_attributes(instance)
        link_html_tag_attributes = format_html_join(' ', '{0}="{1}"',
            [(attr, val) for attr, val in link_attributes.items() if val]
        )
        context.update(dict(instance=instance, placeholder=placeholder, src=src,
                            link_html_tag_attributes=link_html_tag_attributes))
        return context

plugin_pool.register_plugin(TextImagePlugin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-10-09 19:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
from cmsplugin_cascade.models import CascadeElement, CascadePage, IconFont


def forwards(apps, schema_editor):
    for cascade_element in CascadeElement.objects.all():
        if cascade_element.plugin_type not in ['FramedIconPlugin', 'TextIconPlugin']:
            continue

        icon_font = cascade_element.glossary.get('icon_font')
        if icon_font:
            try:
                icon_font = IconFont.objects.get(pk=icon_font)
            except IconFont.DoesNotExist:
                pass
            else:
                cms_page = cascade_element.page.get_public_object()
                defaults = {'icon_font': icon_font}
                public_extension, _ = CascadePage.objects.update_or_create(
                    extended_object=cms_page,
                    public_extension=None,
                    defaults=defaults,
                )
                CascadePage.objects.update_or_create(
                    extended_object=cms_page.get_draft_object(),
                    public_extension=public_extension,
                    defaults=defaults,
                )
                # TODO: After implementing a backward migration, remove deprecated `icon_font` from glossary
                # cascade_element.glossary.pop('icon_font')
                # cascade_element.save()


def backwards(apps, schema_editor):
    print("Backward migration not implemented")


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_cascade', '0019_verbose_table_names'),
    ]

    operations = [
        migrations.AddField(
            model_name='cascadepage',
            name='icon_font',
            field=models.ForeignKey(blank=True, help_text='Set Icon Font globally for this page', null=True, on_delete=django.db.models.deletion.CASCADE, to='cmsplugin_cascade.IconFont', verbose_name='Icon Font'),
        ),
        migrations.RunPython(forwards, reverse_code=backwards),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from collections import OrderedDict

from django.conf import settings
from django.forms import widgets
from django.forms.fields import CharField
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_text

from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.fields import GlossaryField
from cmsplugin_cascade.link.config import LinkPluginBase, LinkElementMixin, LinkForm
from cmsplugin_cascade.link.forms import TextLinkFormMixin
if 'cmsplugin_cascade.icon' in settings.INSTALLED_APPS:
    from cmsplugin_cascade.icon.mixins import IconPluginMixin
else:
    from cmsplugin_cascade.plugin_base import CascadePluginMixinBase as IconPluginMixin


class ButtonTypeWidget(widgets.RadioSelect):
    """
    Render sample buttons in different colors in the button's backend editor.
    """
    BUTTON_TYPES = OrderedDict([
        ('btn-primary', _("Primary")),
        ('btn-secondary', _("Secondary")),
        ('btn-success', _("Success")),
        ('btn-danger', _("Danger")),
        ('btn-warning', _("Warning")),
        ('btn-info', _("Info")),
        ('btn-light', _("Light")),
        ('btn-dark', _("Dark")),
        ('btn-link', _("Link")),
        ('btn-outline-primary', _("Primary")),
        ('btn-outline-secondary', _("Secondary")),
        ('btn-outline-success', _("Success")),
        ('btn-outline-danger', _("Danger")),
        ('btn-outline-warning', _("Warning")),
        ('btn-outline-info', _("Info")),
        ('btn-outline-light', _("Light")),
        ('btn-outline-dark', _("Dark")),
        ('btn-outline-link', _("Link")),
    ])
    template_name = 'cascade/forms/widgets/button_types.html'

    @classmethod
    def get_instance(cls):
        return cls(choices=[(k, v) for k, v in cls.BUTTON_TYPES.items()])


class ButtonSizeWidget(widgets.RadioSelect):
    """
    Render sample buttons in different sizes in the button's backend editor.
    """
    BUTTON_SIZES = OrderedDict([
        ('btn-lg', _("Large button")),
        ('', _("Default button")),
        ('btn-sm', _("Small button")),
    ])
    template_name = 'cascade/forms/widgets/button_sizes.html'

    @classmethod
    def get_instance(cls):
        return cls(choices=[(k, v) for k, v in cls.BUTTON_SIZES.items()])


class BootstrapButtonMixin(IconPluginMixin):
    require_parent = True
    parent_classes = ('BootstrapColumnPlugin', 'SimpleWrapperPlugin',)
    render_template = 'cascade/bootstrap4/button.html'
    allow_children = False
    default_css_class = 'btn'
    default_css_attributes = ['button_type', 'button_size', 'button_options', 'quick_float', 'stretched_link']
    ring_plugin = 'ButtonMixin'
    require_icon_font = False

    button_type = GlossaryField(
        ButtonTypeWidget.get_instance(),
        label=_("Button Type"),
        initial='btn-primary',
        help_text=_("Display Link using this Button Style")
    )

    button_size = GlossaryField(
        ButtonSizeWidget.get_instance(),
        label=_("Button Size"),
        initial='',
        help_text=_("Display Link using this Button Size")
    )

    button_options = GlossaryField(
        widgets.CheckboxSelectMultiple(choices=[
            ('btn-block', _('Block level')),
            ('disabled', _('Disabled')),
        ]),
        label=_("Button Options"),
    )

    quick_float = GlossaryField(
        widgets.RadioSelect(choices=[
            ('', _("Do not float")),
            ('pull-left', _("Pull left")),
            ('pull-right', _("Pull right")),
        ]),
        label=_("Quick Float"),
        initial='',
        help_text=_("Float the button to the left or right.")
    )

    icon_align = GlossaryField(
        widgets.RadioSelect(choices=[
            ('', _("No Icon")),
            ('icon-left', _("Icon placed left")),
            ('icon-right', _("Icon placed right")),
        ]),
        label=_("Icon alignment"),
        initial='',
        help_text=_("Add an Icon before or after the button content.")
    )

    symbol = GlossaryField(
        widgets.HiddenInput(),
        label=_("Select Symbol"),
    )

    stretched_link = GlossaryField(
        widgets.CheckboxInput(),
        label=_("Stretched link"),
        help_text=_("Stretched-link utility to make any anchor the size of it’s nearest position:\
         relative parent, perfect for entirely clickable cards!")
    )

    class Media:
        js = ['cascade/js/admin/buttonmixin.js']

    def render(self, context, instance, placeholder):
        context = super(BootstrapButtonMixin, self).render(context, instance, placeholder)
        try:
            icon_font = self.get_icon_font(instance)
            symbol = instance.glossary.get('symbol')
        except AttributeError:
            icon_font, symbol = None, None
        if icon_font and symbol:
            context['stylesheet_url'] = icon_font.get_stylesheet_url()
            mini_template = '{0}<i class="icon-{1} {2}" aria-hidden="true"></i>{3}'
            icon_align = instance.glossary.get('icon_align')
            if icon_align == 'icon-left':
                context['icon_left'] = format_html(mini_template, '', symbol, 'cascade-icon-left', ' ')
            elif icon_align == 'icon-right':
                context['icon_right'] = format_html(mini_template, ' ', symbol, 'cascade-icon-right', '')
        return context


class BootstrapButtonPlugin(BootstrapButtonMixin, LinkPluginBase):
    module = 'Bootstrap'
    name = _("Button")
    model_mixins = (LinkElementMixin,)
    fields = ['link_content'] + list(LinkPluginBase.fields)
    glossary_field_order = ['button_type', 'button_size', 'button_options', 'quick_float',
                            'target', 'title', 'icon_align', 'icon_font', 'symbol', 'stretched_link']
    ring_plugin = 'ButtonPlugin'
    DEFAULT_BUTTON_ATTRIBUTES = {'role': 'button'}

    class Media:
        css = {'all': ['cascade/css/admin/bootstrap4-buttons.css', 'cascade/css/admin/iconplugin.css']}
        js = ['cascade/js/admin/buttonplugin.js']

    @classmethod
    def get_identifier(cls, obj):
        identifier = super(BootstrapButtonPlugin, cls).get_identifier(obj)
        content = obj.glossary.get('link_content')
        if not content:
            try:
                content = force_text(ButtonTypeWidget.BUTTON_TYPES[obj.glossary['button_type']])
            except KeyError:
                content = _("Empty")
        return format_html('{}{}', identifier, content)

    def get_form(self, request, obj=None, **kwargs):
        link_content = CharField(
            required=False,
            label=_("Button Content"),
            widget=widgets.TextInput(attrs={'size': 50}),
        )
        Form = type(str('ButtonForm'), (TextLinkFormMixin, getattr(LinkForm, 'get_form_class')(),),
                    {'link_content': link_content})
        kwargs.update(form=Form)
        return super(BootstrapButtonPlugin, self).get_form(request, obj, **kwargs)

    @classmethod
    def get_css_classes(cls, obj):
        css_classes = cls.super(BootstrapButtonPlugin, cls).get_css_classes(obj)
        if obj.glossary.get('stretched_link'):
            css_classes.append('stretched_link')
        return css_classes

    @classmethod
    def get_html_tag_attributes(cls, obj):
        attributes = cls.super(BootstrapButtonPlugin, cls).get_html_tag_attributes(obj)
        attributes.update(cls.DEFAULT_BUTTON_ATTRIBUTES)
        return attributes

plugin_pool.register_plugin(BootstrapButtonPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.forms import MediaDefiningClass, widgets
from django.utils import six
from django.utils.encoding import python_2_unicode_compatible
from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _
from cmsplugin_cascade.fields import GlossaryField
from cmsplugin_cascade.bootstrap4.grid import Breakpoint


@python_2_unicode_compatible
class BootstrapUtilitiesMixin(six.with_metaclass(MediaDefiningClass)):
    """
    If a Cascade plugin is listed in ``settings.CMSPLUGIN_CASCADE['plugins_with_extra_mixins']``,
    then this ``BootstrapUtilsMixin`` class is added automatically to its plugin class in order to
    enrich it with utility classes offered by Bootstrap-4.
    """
    def __str__(self):
        return self.plugin_class.get_identifier(self)

    @classmethod
    def get_css_classes(cls, obj):
        """Enrich list of CSS classes with customized ones"""
        css_classes = super(BootstrapUtilitiesMixin, cls).get_css_classes(obj)
        for utility_field_name in cls.utility_field_names:
            css_classes.append(obj.glossary.get(utility_field_name))
        return css_classes


class BootstrapUtilities(type):
    """
    Factory for building a class ``BootstrapUtilitiesMixin``. This class then is used as a mixin to
    all sorts of Bootstrap-4 plugins. Various Bootstrap-4 plugins are shipped using this mixin class
    in different configurations. These configurations can be overridden through the project's
    settings using:
    ```
    CMSPLUGIN_CASCADE['plugins_with_extra_mixins'] = {'Bootstrap<ANY>Plugin': BootstrapUtilities(
        BootstrapUtilities.background_and_color,
        BootstrapUtilities.margins,
        BootstrapUtilities.paddings,
        …
    )
    ```
    or similar.
    The class ``BootstrapUtilities`` offers a bunch of property methods which return a list of
    input fields and/or select boxes. They then can be added to the plugin's editor. This is
    specially useful to add CSS classes from the utilities section of Bootstrap-4, such as
    margins, borders, colors, etc.
    """
    def __new__(cls, *args):
        glossary_fields = []
        for arg in args:
            if isinstance(arg, property):
                arg = arg.fget(cls)
            if isinstance(arg, (list, tuple)):
                glossary_fields.extend([gf for gf in arg if isinstance(gf, GlossaryField)])
            elif isinstance(arg, GlossaryField):
                glossary_fields.append(arg)
        attrs = {'glossary_fields': glossary_fields, 'utility_field_names': [gf.name for gf in glossary_fields]}
        return type(BootstrapUtilitiesMixin.__name__, (BootstrapUtilitiesMixin,), attrs)

    @property
    def background_and_color(cls):
        choices = [
            ('', _("Default")),
            ('bg-primary text-white', _("Primary with white text")),
            ('bg-secondary text-white', _("Secondary with white text")),
            ('bg-success text-white', _("Success with white text")),
            ('bg-danger text-white', _("Danger with white text")),
            ('bg-warning text-white', _("Warning with white text")),
            ('bg-info text-white', _("Info with white text")),
            ('bg-light text-dark', _("Light with dark text")),
            ('bg-dark text-white', _("Dark with white text")),
            ('bg-white text-dark', _("White with dark text")),
            ('bg-transparent text-dark', _("Transparent with dark text")),
            ('bg-transparent text-white', _("Transparent with white text")),
        ]
        return GlossaryField(
            widgets.Select(choices=choices),
            label=_("Background and color"),
            name='background_and_color',
            initial=''
        )

    @property
    def margins(cls):
        glossary_fields = []
        choices_format = [
            ('m-{}{}', _("4 sided margins ({})")),
            ('mx-{}{}', _("Horizontal margins ({})")),
            ('my-{}{}', _("Vertical margins ({})")),
            ('mt-{}{}', _("Top margin ({})")),
            ('mr-{}{}', _("Right margin ({})")),
            ('mb-{}{}', _("Bottom margin ({})")),
            ('ml-{}{}', _("Left margin ({})")),
        ]
        sizes = list(range(0, 6)) + ['auto']
        for bp in Breakpoint.range(Breakpoint.xs, Breakpoint.xl):
            if bp == Breakpoint.xs:
                choices = [(c.format('', s), format_lazy(l, s)) for c, l in choices_format for s in sizes]
                choices.insert(0, ('', _("No Margins")))
            else:
                choices = [(c.format(bp.name + '-', s), format_lazy(l, s)) for c, l in choices_format for s in sizes]
                choices.insert(0, ('', _("Inherit from above")))
            glossary_fields.append(GlossaryField(
                widgets.Select(choices=choices),
                label=format_lazy(_("Margins for {breakpoint}"), breakpoint=bp.label),
                name='margins_{}'.format(bp.name),
                initial=''
            ))
        return glossary_fields

    @property
    def paddings(cls):
        glossary_fields = []
        choices_format = [
            ('p-{}{}', _("4 sided padding ({})")),
            ('px-{}{}', _("Horizontal padding ({})")),
            ('py-{}{}', _("Vertical padding ({})")),
            ('pt-{}{}', _("Top padding ({})")),
            ('pr-{}{}', _("Right padding ({})")),
            ('pb-{}{}', _("Bottom padding ({})")),
            ('pl-{}{}', _("Left padding ({})")),
        ]
        sizes = range(0, 6)
        for bp in Breakpoint.range(Breakpoint.xs, Breakpoint.xl):
            if bp == Breakpoint.xs:
                choices = [(c.format('', s), format_lazy(l, s)) for c, l in choices_format for s in sizes]
                choices.insert(0, ('', _("No Padding")))
            else:
                choices = [(c.format(bp.name + '-', s), format_lazy(l, s)) for c, l in choices_format for s in sizes]
                choices.insert(0, ('', _("Inherit from above")))
            glossary_fields.append(GlossaryField(
                widgets.Select(choices=choices),
                label=format_lazy(_("Padding for {breakpoint}"), breakpoint=bp.label),
                name='padding_{}'.format(bp.name),
                initial=''
            ))
        return glossary_fields



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import io
import mimetypes
import os

from django.conf import settings
from django.conf.urls import url
from django.core.exceptions import ViewDoesNotExist
from django.http.response import HttpResponse
from django.views.generic import TemplateView
from django.utils.cache import patch_cache_control
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


class SphinxDocsView(TemplateView):
    def get(self, request, *args, **kwargs):
        slug = kwargs.get('slug', '')
        _, extension = os.path.splitext(slug)
        if extension in ['.png', '.jpg', '.jpeg', '.gif']:
            filename = os.path.join(settings.SPHINX_DOCS_ROOT, slug)
            content_type, _ = mimetypes.guess_type(filename)
            with io.open(filename, 'rb') as fd:
                response = HttpResponse(content=fd.read(), content_type=content_type)
                patch_cache_control(response, cache_control='max-age=86400')
                return response
        return super(SphinxDocsView, self).get(request, page=slug, *args, **kwargs)

    def get_template_names(self):
        return [self.request.current_page.get_template()]

    def get_context_data(self, page='index.html', **kwargs):
        context = super(SphinxDocsView, self).get_context_data(**kwargs)
        filename = os.path.join(settings.SPHINX_DOCS_ROOT, page, 'index.html')
        if not os.path.exists(filename):
            raise ViewDoesNotExist("{} does not exist".format(page))
        with io.open(filename, encoding='utf-8') as fd:
            context.update(page_content=mark_safe(fd.read()))
        return context


@apphook_pool.register
class SphinxDocsApp(CMSApp):
    name = _("Sphinx Documentation")

    def get_urls(self, page=None, language=None, **kwargs):
        return [
            url(r'^(?P<slug>\S+)/$', SphinxDocsView.as_view(), name='sphinx-documentation'),
        ]
